#include "track.lsl"
#include "marker.lsl"

#define mix(min, max, value) (((max) - (min)) * (value) + (min))

float clamp(float min, float max, float value)
{
  if (value < min)
    return min;
  if (value > max)
    return max;

  return value;
}


vector world2Map(vector worldPosition)
{
  float worldBottom  = 259574.78;
  float worldTop     = 259913.31;
  float worldLeft    = 259291.14;
  float worldRight   = 259580.29;
  float worldUp      = 800;
  float worldDown    = 20;

  vector size = llGetScale();

  float mapBottom  = -size.y/2.0;
  float mapTop     = size.y/2.0;
  float mapLeft    = size.x/2.0;
  float mapRight   = -size.x/2.0;
  float mapUp      = size.x * 2 + size.z/2.0;
  float mapDown    = size.z/2.0;

  float x;
  x = (worldPosition.x - worldLeft) / (worldRight - worldLeft);
  x = clamp(0, 1, x);
  x = mix(mapRight, mapLeft, x);

  float y;
  y = (worldPosition.y - worldBottom) / (worldTop - worldBottom);
  y = clamp(0, 1, y);
  y = mix(mapBottom, mapTop, y);


  float z;
  z = (worldPosition.z - worldDown) / (worldUp - worldDown);
  z = clamp(0, 1, z);
  z = mix(mapDown, mapUp, z);

  return <x, y, z>;
}


vector type2Color(string type)
{
  if (type == "friend")
    return <0.2, 0.2, 1>;

  if (type == "foe")
    return <1, 0.2, 0.2>;

  else
    return <0.2, 1, 0.2>;
}



// track requires these callbacks:
trackAddCallback(string id, string name, vector position, string type)
{
  vector mapPosition = world2Map(position);
  vector color = type2Color(type);

  markerAdd(id, name, mapPosition, color);
}

trackSetCallback(string id, string name, vector position, string type)
{
  vector mapPosition = world2Map(position);
  vector color = type2Color(type);

  markerSet(id, name, mapPosition, color);
}

trackRemoveCallback(string id)
{
  markerRemove(id);
}



default
{
  state_entry()
    {
      llSetTimerEvent(10);
      initTrackCommand();
      initMarker();
      llOwnerSay("my email address is " + (string)llGetKey() + "@lsl.secondlife.com");
    }

  on_rez(integer n)
    {
      llOwnerSay("my email address is " + (string)llGetKey() + "@lsl.secondlife.com");
      initTrackCommand();
    }

  listen(integer channel, string name, key id,  string message)
    {
      listenTrackCommand(channel, name, id, message);
    }

  email(string time, string address, string subj, string message, integer num_left)
    {
      emailTrackCommand(time, address, subj, message, num_left);
    }

  timer()
    {
      timerTrackCommand();
    }
}
