#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"


#define Q_TRACK_COMMAND_CHANNEL 500
#define Q_TRACK_TIMEOUT (15*60)


#define Q_TRACK_KEY           "-key"
#define Q_TRACK_NAME          "-name"
#define Q_TRACK_POSITION      "-position"
#define Q_TRACK_TYPE          "-type"


list trackCommand;

list trackKeys;
list trackNames;
list trackPositions;
list trackTypes;
list trackTimes;


integer trackFind(string trackKey)
{
  integer i;

  for (i = 0; i < llGetListLength(trackKeys); i++)
    {
      if (llList2String(trackKeys, i) == trackKey)
	return i;
    }
  
  return -1;
}



trackAdd(string trackKey, 
	string trackName, 
	vector trackPosition, 
	string trackType,
	integer trackTime)
{
  trackKeys      += [ trackKey ];
  trackNames     += [ trackName ];
  trackPositions += [ trackPosition ];
  trackTypes     += [ trackType ];
  trackTimes     += [ trackTime ];

  trackAddCallback(trackKey, trackName, trackPosition, trackType);
}



trackSet(integer index, 
	 string trackKey, 
	 string trackName, 
	 vector trackPosition, 
	 string trackType,
	 integer trackTime)
{
  trackKeys      = llListReplaceList(trackKeys, [ trackKey ], index, index);
  trackNames     = llListReplaceList(trackNames, [ trackName ], index, index);
  trackPositions = llListReplaceList(trackPositions, [ trackPosition ], index, index);
  trackTimes     = llListReplaceList(trackTimes, [ trackTime ], index, index);

  trackSetCallback(trackKey, trackName, trackPosition, trackType);
}



trackRemove(integer index)
{
  string trackKey = llList2String(trackKeys, index);

  trackKeys      = llDeleteSubList(trackKeys, index, index);
  trackNames     = llDeleteSubList(trackNames, index, index);
  trackPositions = llDeleteSubList(trackPositions, index, index);
  trackTimes     = llDeleteSubList(trackTimes, index, index);

  trackRemoveCallback(trackKey);
}


trackCleanTable()
{
  integer i = 0;
  
  while (i < llGetListLength(trackKeys))
    {
      integer time = llGetUnixTime();
      integer trackTime = llList2Integer(trackTimes, i);

      if (llAbs(time - trackTime) > Q_TRACK_TIMEOUT)
	trackRemove(i);
      else
	i++;
    }
}



track(string trackKey, string trackName, vector trackPosition, string trackType)
{
  integer trackTime = llGetUnixTime();

  integer index = trackFind(trackKey);

  if (index != -1)
    {
      trackSet(index, trackKey, trackName, trackPosition, trackType, trackTime);
    }

  else
    trackAdd(trackKey, trackName, trackPosition, trackType, trackTime);
}



buildTrackCommand()
{
  trackCommand = qCommandCreate("track");
  trackCommand = qCommandAddUniversalSwitches(trackCommand);

  trackCommand = qCommandAddSwitch(trackCommand, [Q_TRACK_KEY, 
						  "string"]);
  trackCommand = qCommandAddSwitch(trackCommand, [Q_TRACK_NAME, 
						  "string"]);
  trackCommand = qCommandAddSwitch(trackCommand, [Q_TRACK_POSITION, 
						  "float", "float", "float"]);
  trackCommand = qCommandAddSwitch(trackCommand, [Q_TRACK_TYPE, 
						  "string"]);
}


 
initTrackCommand()
{
  buildTrackCommand();

  llListen(Q_TRACK_COMMAND_CHANNEL, "", NULL_KEY, "");

  integer privateChannel = llGetStartParameter();
  if (privateChannel != 0)
    llListen(privateChannel, "", NULL_KEY, "");

  llGetNextEmail("", "");
}


 
executeTrackCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  string trackKey;
  string trackName;
  vector trackPosition;
  string trackType;

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_TRACK_KEY)
	{
	  trackKey = llList2String(switches, i+1);
	  i+=2;
	}

      else if (switch == Q_TRACK_NAME)
	{
	  trackName = llList2String(switches, i+1);
	  i+=2;
	}

      else if (switch == Q_TRACK_POSITION)
	{
	  trackPosition = ( < llList2Float(switches, i+1),
			    llList2Float(switches, i+2),
			    llList2Float(switches, i+3) > );
	    
	  i+=4;
	}

      else if (switch == Q_TRACK_TYPE)
	{
	  trackType = llList2String(switches, i+1);
	  i+=2;
	}

      else
	i = qCommandExecuteUniversalSwitch(switches, i);
    }


  qDebug("track " + trackKey + " " + trackName + " " + (string) trackPosition, 10);

  track(trackKey, trackName, trackPosition, trackType);

}



listenTrackCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands on the public channel from our owner
  if ((channel == Q_TRACK_COMMAND_CHANNEL) &&
      (id != llGetOwner()))
    return;

  list switches = qCommandParseText(trackCommand, message);

  if (switches != [])
    executeTrackCommand(switches);
}


emailTrackCommand(string time, string address, 
		  string subj, string message, integer num_left)
{
  string messageContent;

  if (llGetSubString(message, 0, 10) == "Object-Name")
    messageContent = llDeleteSubString(message, 0, 
				       llSubStringIndex(message, "\n\n") + 1);
  else
    messageContent = message;

  qDebug("email received: " + messageContent, 10);

  list switches = qCommandParseText(trackCommand, messageContent);

  if (switches != [])
    executeTrackCommand(switches);

  llGetNextEmail("", "");
}



timerTrackCommand()
{
  llGetNextEmail("", "");
  trackCleanTable();
}
