#include "qLab/qMath.lsl"
#include "track.lsl"
#include "marker.lsl"

vector world2Map(vector worldPosition)
{
  float worldBottom  = 974 * 256;
  float worldTop     = 1041 * 256;
  float worldLeft    = 986 * 256;
  float worldRight   = 1044 * 256;

  vector size = llGetScale();

  float mapBottom  = -size.y/2.0;
  float mapTop     = size.y/2.0;
  float mapLeft    = size.x/2.0;
  float mapRight   = -size.x/2.0;

  float x;
  x = (worldPosition.x - worldLeft) / (worldRight - worldLeft);
  x = qClamp(0, 1, x);
  x = qMix(mapRight, mapLeft, x);

  float y;
  y = (worldPosition.y - worldBottom) / (worldTop - worldBottom);
  y = qClamp(0, 1, y);
  y = qMix(mapBottom, mapTop, y);

  return <x, y, size.z / 2.0>;
}


vector type2Color(string type)
{
  if (type == "friend")
    return <0.2, 0.2, 1>;

  if (type == "foe")
    return <1, 0.2, 0.2>;

  else
    return <0.2, 1, 0.2>;
}



// track requires these callbacks:
trackAddCallback(string id, string name, vector position, string type)
{
  vector mapPosition = world2Map(position);
  vector color = type2Color(type);

  markerAdd(id, name, mapPosition, color);
}

trackSetCallback(string id, string name, vector position, string type)
{
  vector mapPosition = world2Map(position);
  vector color = type2Color(type);

  markerSet(id, name, mapPosition, color);
}

trackRemoveCallback(string id)
{
  markerRemove(id);
}



default
{
  state_entry()
    {
      llSetTimerEvent(30);
      initTrackCommand();
      initMarker();
      llOwnerSay("my email address is " + (string)llGetKey() + "@lsl.secondlife.com");
    }

  on_rez(integer n)
    {
      llOwnerSay("my email address is " + (string)llGetKey() + "@lsl.secondlife.com");
      initTrackCommand();
    }

  listen(integer channel, string name, key id,  string message)
    {
      listenTrackCommand(channel, name, id, message);
    }

  email(string time, string address, string subj, string message, integer num_left)
    {
      emailTrackCommand(time, address, subj, message, num_left);
    }

  timer()
    {
      timerTrackCommand();
    }
}
